from django.utils.translation import ugettext_lazy as _
from django.contrib import admin
from django.conf import settings
from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase
from simplegallery.models import GalleryPublication, CarouselImage,\
    CarouselFeature, Gallery
from django.template.defaultfilters import title

class SimpleGalleryPublicationPlugin(CMSPluginBase):
    model = GalleryPublication
    name = _("Gallery")
    render_template = "simplegallery/gallery_plugin.html"
    change_form_template = "simplegallery/plugin_form.html"
    text_enabled = False
    
    if not getattr(settings, 'CMSPLUGIN_SIMPLE_GALLERY_STYLE_CHOICES', False):
        exclude = ('style',)
    
    def render(self, context, instance, placeholder):
        context.update({
            'instance': instance,
            'gallery': instance.gallery,
            'images': instance.gallery.images.all(),
            'image_size': get_image_size(context, instance),
            'placeholder': placeholder,
        })
        return context
        
    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        """
        Get a form Field for a ForeignKey.
        """
        gallery = self.model.gallery.field
        if request and db_field is gallery and not request.user.is_superuser:
            kwargs['queryset'] = Gallery.objects.filter(groups__in=request.user.groups.all()).distinct()
        else:
            kwargs['queryset'] = Gallery.objects.all()
        return super(SimpleGalleryPublicationPlugin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    """
    class PluginMedia:
        css = {
            'all': ('%ssimplegallery/css/simplegallery.css' % settings.MEDIA_URL,)
        }
    """
 
plugin_pool.register_plugin(SimpleGalleryPublicationPlugin)

class CarouselImageInline(admin.StackedInline):
    model = CarouselImage
    fk_name = 'carousel_feature'
    extra = 1

class CarouselFeaturePlugin(CMSPluginBase):
    model = CarouselFeature
    name = title(_('Gallery lite'))
    placeholders = ('feature_home',)
    render_template = "simplegallery/gallery_lite.html"
    change_form_template = "simplegallery/plugin_form.html"
    text_enabled = False
    inlines = [
        CarouselImageInline,
    ]
    
    def render(self, context, instance, placeholder):
        context.update({
            'instance': instance,
            'images': instance.images.all(),
            'image_size': get_image_size(context, instance),
            'placeholder': placeholder,
        })
        return context
    
    class PluginMedia:
        '''
        js = ('%ssimplegallery/js/jquery.cycle.min.js'% settings.MEDIA_URL,
              '%ssimplegallery/js/jquery.cycle.trans.min.js'% settings.MEDIA_URL,)
        '''
plugin_pool.register_plugin(CarouselFeaturePlugin)

def get_image_size(context, instance):
    placeholder_width = context.get('width', None)
    if placeholder_width:
        width = placeholder_width
    else:
        try:
            width = instance.image.width
        except:
            width = 100
    height = int(float(width) / instance.get_aspect_ratio())
    return (u'%sx%s' % (width, height), width, height)


from django.db import models
from django.forms.fields import RegexField
from django.utils.translation import ugettext_lazy
from django.utils.encoding import smart_unicode
from django.core import exceptions
import re

ASPECT_RATIO_RE = re.compile('[1-9]\d*:[1-9]\d*')


class AspectRatioFormField(RegexField):
    def __init__(self, *args, **kwargs):
        super(AspectRatioFormField, self).__init__(*args, **kwargs)
        self.localize = True
    
    def localize_value(self, value):
        if value:
            return '%s:%s' % tuple(value)
        return value
 

class AspectRatioField(models.CharField):
    __metaclass__ = models.SubfieldBase
    
    def to_python(self, value):
        if isinstance(value, list):
            return value
        if isinstance(value, basestring):
            if ':' in value:
                return [int(x) for x in value.split(':')]
            else:
                return None
        if value is None:
            if self.null:
                return value
            else:
                raise exceptions.ValidationError(
                    ugettext_lazy("This field cannot be null."))
        return self.to_python(smart_unicode(value))

    def get_prep_value(self, value):
        if isinstance(value, list):
            return '%s:%s' % tuple(value)
        if isinstance(value, basestring):
            if ':' in value:
                return value
        return None
    
    def get_db_prep_value(self, value, connection, prepared=False):
        return self.get_prep_value(value)

    def value_from_object(self, obj):
        return self.to_python(getattr(obj, self.attname))

    def formfield(self, **kwargs):
        defaults = {'form_class': AspectRatioFormField, 'regex': ASPECT_RATIO_RE}
        if 'initial' in kwargs:
            kwargs['initial'] = self.get_prep_value(kwargs['initial'])
        defaults.update(kwargs)
        return super(AspectRatioField, self).formfield(**defaults)
    
    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect ourselves, since we inherit.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.CharField"
        args, kwargs = introspector(self)
        # That's our definition!
        return (field_class, args, kwargs)


from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from django.contrib.auth.models import Group
from cms.models import Page
from cms.models import CMSPlugin
from cms.models.fields import PageField
from django.utils.text import truncate_words
from multilingual.translation import TranslationModel

from filer.fields.image import FilerImageField

from simplegallery.fields import AspectRatioField

from tinymce.models import HTMLField
from simplegallery import south_introspections # make sure south knows about the HTMLField

CMSPLUGIN_SIMPLE_GALLERY_STYLE_CHOICES = getattr( settings, 'CMSPLUGIN_SIMPLE_GALLERY_STYLE_CHOICES',() )

class Gallery(models.Model):
    name = models.CharField(max_length=255, unique=True,
        help_text=_("A unique identifier for this gallery, this will only be used in the admin panel."))
    groups = models.ManyToManyField(Group, blank=True)
    
    class Translation(TranslationModel):
        title = models.CharField(_('title'), max_length=255, blank=True, default='')
        description = HTMLField(blank=True, default='')

    class Meta:
        verbose_name = _('gallery')
        verbose_name_plural = _('galleries')

    def __unicode__(self):
        return self.name
    
    def has_drop_up(self):
        if not hasattr(self, '_has_drop_up'):
            self._has_drop_up = bool(self.images.filter(drop_up_links__isnull=False).count())
        return self._has_drop_up
        

class Image(models.Model):
    gallery = models.ForeignKey(Gallery, related_name="images")
    image = FilerImageField()
    page_link = PageField(verbose_name=_('page link'), null=True, blank=True)
    free_link = models.CharField(_("link"), max_length=2048, blank=True, null=True, 
                                 help_text=_("an absolute url"))
    ordering = models.IntegerField(null=True, blank=True)
    drop_up_links = models.ManyToManyField('sites.Site', blank=True)
    
    class Translation(TranslationModel):
        title = models.CharField(_('title'), max_length=255, blank=True, default='')
        description = HTMLField(_('description'), blank=True, default='')

    class Meta:
        ordering = ('ordering',)
        verbose_name = _('image')
        verbose_name_plural = _('images')

    def __unicode__(self):
        if isinstance(self.title, basestring):
            return self.title
        return unicode(self.pk)

    @property
    def file(self):
        return self.image.file
    
    @property
    def link(self):
        if self.free_link:
            return self.free_link
        elif self.page_link and self.page_link:
            return self.page_link.get_absolute_url()
        else:
            return ''
    
aspect_ratio_choices = getattr(settings, "IMAGE_ASPECT_RATIO_CHOICES", (
    (1, '1:1'),
    (1.33333, '4:3'),
    (1.77777, '16:9'),
))


class GalleryPublication(CMSPlugin):
    gallery = models.ForeignKey(Gallery)
    interval = models.PositiveSmallIntegerField(_('interval'), default=0)
    style = models.CharField(_("gallery style"), max_length=255, blank=True, choices=CMSPLUGIN_SIMPLE_GALLERY_STYLE_CHOICES)
    aspect_ratio = models.FloatField(_('aspect ratio'), choices=aspect_ratio_choices, default=1)
    raw_aspect_ratio = AspectRatioField(null=True, blank=True, max_length=20)
    
    def get_aspect_ratio(self):
        if self.raw_aspect_ratio:
            x,y = self.raw_aspect_ratio
            return float(x) / float(y)
        return self.aspect_ratio
    
class CarouselImage(models.Model):
    carousel_feature = models.ForeignKey('CarouselFeature', related_name="images")
    image = FilerImageField()
    date = models.DateField(_('date'), null=True, blank=True)
    title = models.CharField(_('title'), max_length=150, null=True, blank=True)
    description = models.TextField(_('description'), null=True, blank=True)
    #title_2 = models.CharField(_('title 2'), max_length=150, null=True, blank=True)
    #description_2 = models.TextField(_('description 2'), null=True, blank=True)
    page_link = PageField(verbose_name=_('page link'), null=True, blank=True)
    free_link = models.CharField(_("link"), max_length=2048, blank=True, null=True, 
                                 help_text=_("an absolute url"))
    url = models.URLField(_('URL'), blank=True, \
        help_text=_('If the %(page_link)s field is not used, you can enter an external URL here.') % {'page_link': _('page link')})
    ordering = models.PositiveSmallIntegerField(_('ordering'), null=True, blank=True)
    
    class Meta:
        ordering = ('ordering',)
        verbose_name = _('carousel image')
        verbose_name_plural = _('carousel images')

    def __unicode__(self):
        return self.title

    @property
    def file(self):
        return self.image.file
    
    def get_url(self):
        if self.page_link:
            return self.page_link.get_absolute_url()
        elif self.url:
            return self.url
    @property
    def link(self):
        if self.free_link:
            return self.free_link
        elif self.page_link and self.page_link:
            return self.page_link.get_absolute_url()
        else:
            return ''
    
class CarouselFeature(CMSPlugin):
    title = models.CharField(_('title'), max_length=50, blank=True)
    interval = models.PositiveSmallIntegerField(_('interval'), default=3)
    aspect_ratio = models.FloatField(_('aspect ratio'), choices=aspect_ratio_choices, default=1)
    
    def __unicode__(self):
        return truncate_words(self.title, 3)[:20] + "..."
    
    def get_aspect_ratio(self):
        return self.aspect_ratio



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Changing field 'GalleryTranslation.title'
        db.alter_column('simplegallery_gallery_translation', 'title', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True))

        # Changing field 'GalleryTranslation.description'
        db.alter_column('simplegallery_gallery_translation', 'description', self.gf('tinymce.models.HTMLField')(blank=True))

        # Changing field 'ImageTranslation.title'
        db.alter_column('simplegallery_image_translation', 'title', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True))

        # Changing field 'ImageTranslation.description'
        db.alter_column('simplegallery_image_translation', 'description', self.gf('tinymce.models.HTMLField')(blank=True))


    def backwards(self, orm):
        
        # Changing field 'GalleryTranslation.title'
        db.alter_column('simplegallery_gallery_translation', 'title', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True))

        # Changing field 'GalleryTranslation.description'
        db.alter_column('simplegallery_gallery_translation', 'description', self.gf('django.db.models.fields.TextField')(null=True, blank=True))

        # Changing field 'ImageTranslation.title'
        db.alter_column('simplegallery_image_translation', 'title', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True))

        # Changing field 'ImageTranslation.description'
        db.alter_column('simplegallery_image_translation', 'description', self.gf('django.db.models.fields.TextField')(null=True, blank=True))


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'publisher_is_draft': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True', 'blank': 'True'}),
            'publisher_public': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'publisher_draft'", 'unique': 'True', 'null': 'True', 'to': "orm['cms.CMSPlugin']"}),
            'publisher_state': ('django.db.models.fields.SmallIntegerField', [], {'default': '0', 'db_index': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.page': {
            'Meta': {'object_name': 'Page'},
            'changed_by': ('django.db.models.fields.CharField', [], {'max_length': '70'}),
            'created_by': ('django.db.models.fields.CharField', [], {'max_length': '70'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_navigation': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True', 'blank': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'limit_visibility_in_menu': ('django.db.models.fields.SmallIntegerField', [], {'default': 'None', 'null': 'True', 'db_index': 'True', 'blank': 'True'}),
            'login_required': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'moderator_state': ('django.db.models.fields.SmallIntegerField', [], {'default': '1', 'blank': 'True'}),
            'navigation_extenders': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '80', 'null': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['cms.Page']"}),
            'placeholders': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['cms.Placeholder']", 'symmetrical': 'False'}),
            'publication_date': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'publication_end_date': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'publisher_is_draft': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True', 'blank': 'True'}),
            'publisher_public': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'publisher_draft'", 'unique': 'True', 'null': 'True', 'to': "orm['cms.Page']"}),
            'publisher_state': ('django.db.models.fields.SmallIntegerField', [], {'default': '0', 'db_index': 'True'}),
            'reverse_id': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'soft_root': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'filer.file': {
            'Meta': {'object_name': 'File'},
            '_file': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            '_file_size': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            '_file_type_plugin_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'folder': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'all_files'", 'null': 'True', 'to': "orm['filer.Folder']"}),
            'has_all_mandatory_data': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'original_filename': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'owned_files'", 'null': 'True', 'to': "orm['auth.User']"}),
            'uploaded_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        'filer.folder': {
            'Meta': {'unique_together': "(('parent', 'name'),)", 'object_name': 'Folder'},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'modified_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'filer_owned_folders'", 'null': 'True', 'to': "orm['auth.User']"}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['filer.Folder']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'uploaded_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        'filer.image': {
            'Meta': {'object_name': 'Image', '_ormbases': ['filer.File']},
            '_height': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            '_width': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'author': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'date_taken': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'default_alt_text': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'default_caption': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'file_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['filer.File']", 'unique': 'True', 'primary_key': 'True'}),
            'must_always_publish_author_credit': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'must_always_publish_copyright': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'subject_location': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '64', 'null': 'True', 'blank': 'True'})
        },
        'simplegallery.carouselfeature': {
            'Meta': {'object_name': 'CarouselFeature', 'db_table': "'cmsplugin_carouselfeature'", '_ormbases': ['cms.CMSPlugin']},
            'aspect_ratio': ('django.db.models.fields.FloatField', [], {'default': '1'}),
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'interval': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '3'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'})
        },
        'simplegallery.carouselimage': {
            'Meta': {'object_name': 'CarouselImage'},
            'carousel_feature': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'images'", 'to': "orm['simplegallery.CarouselFeature']"}),
            'date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'description_2': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['filer.Image']"}),
            'ordering': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'page_link': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Page']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'title_2': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        'simplegallery.gallery': {
            'Meta': {'object_name': 'Gallery'},
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'simplegallery.gallerypublication': {
            'Meta': {'object_name': 'GalleryPublication', 'db_table': "'cmsplugin_gallerypublication'", '_ormbases': ['cms.CMSPlugin']},
            'aspect_ratio': ('django.db.models.fields.FloatField', [], {'default': '1'}),
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'gallery': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['simplegallery.Gallery']"}),
            'interval': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'raw_aspect_ratio': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'style': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'})
        },
        'simplegallery.gallerytranslation': {
            'Meta': {'unique_together': "(('language_code', 'master'),)", 'object_name': 'GalleryTranslation', 'db_table': "'simplegallery_gallery_translation'"},
            'description': ('tinymce.models.HTMLField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['simplegallery.Gallery']"}),
            'title': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'})
        },
        'simplegallery.image': {
            'Meta': {'object_name': 'Image'},
            'drop_up_links': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['sites.Site']", 'symmetrical': 'False', 'blank': 'True'}),
            'gallery': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'images'", 'to': "orm['simplegallery.Gallery']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['filer.Image']"}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'page_link': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Page']", 'null': 'True', 'blank': 'True'})
        },
        'simplegallery.imagetranslation': {
            'Meta': {'unique_together': "(('language_code', 'master'),)", 'object_name': 'ImageTranslation', 'db_table': "'simplegallery_image_translation'"},
            'description': ('tinymce.models.HTMLField', [], {'default': "''", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '15', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['simplegallery.Image']"}),
            'title': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'})
        },
        'sites.site': {
            'Meta': {'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['simplegallery']



from south.modelsinspector import add_introspection_rules
from tinymce.models import HTMLField
from django.conf import settings
from django.db.models.fields import NOT_PROVIDED


rules = [
            (
                (HTMLField, ),
                [],
                {"null": ["null", {"default": False}],
                "blank": ["blank", {"default": False, "ignore_if":"primary_key"}],
                "primary_key": ["primary_key", {"default": False}],
                "max_length": ["max_length", {"default": None}],
                "unique": ["_unique", {"default": False}],
                "db_index": ["db_index", {"default": False}],
                "default": ["default", {"default": NOT_PROVIDED}],
                "db_column": ["db_column", {"default": None}],
                "db_tablespace": ["db_tablespace", {"default": settings.DEFAULT_INDEX_TABLESPACE}],
                },
            ),
        ]

add_introspection_rules(rules, ["^tinymce\.models\.HTMLField$"])


APP_NAME = 'simplegallery'
PACKAGE_NAME = 'django-%s' % APP_NAME
DESCRIPTION = 'django image gallery app'
PROJECT_URL = 'http://github.com/divio/%s/' % PACKAGE_NAME

INSTALL_REQUIRES = [
    'django (<1.2.5)',
    'mptt (<0.4)',
    'django_cms',
    'django_filer',
    'django_multilingual_ng',
    'django_tinymce',
] # e.g 'django (>1.1.0)'

EXTRA_REQUIRES={
    'South':  ["south"],
}


AUTHOR="Stefan Foulis"

EXTRA_CLASSIFIERS = [
]


# DO NOT EDIT ANYTHING DOWN HERE... this should be common to all django app packages
from setuptools import setup, find_packages
import os

version = __import__(APP_NAME).__version__

classifiers = [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]
if not 'a' in version and not 'b' in version: classifiers.append('Development Status :: 5 - Production/Stable')
elif 'a' in version: classifiers.append('Development Status :: 3 - Alpha')
elif 'b' in version: classifiers.append('Development Status :: 4 - Beta')

for c in EXTRA_CLASSIFIERS:
    if not c in classifiers:
        classifiers.append(c)

media_files = []
for dir in ['%s/media' % APP_NAME,'%s/templates' % APP_NAME]:
    for dirpath, dirnames, filenames in os.walk(dir):
        media_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])
#media_files.append('README.rst')
#media_files.append('HISTORY')
# build the MANIFEST.in file
"""
open(os.path.join(os.path.dirname(__file__), 'MANIFEST.in'), 'w').write('''include README.rst
include HISTORY
recursive-include %s/media *
recursive-include %s/templates *
''')
"""
    


def read(fname):
    # read the contents of a text file
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    author=AUTHOR,
    name=PACKAGE_NAME,
    version=version,
    url=PROJECT_URL,
    description=DESCRIPTION,
    long_description=read('README.rst') + '\n\n\n' + read('HISTORY'),
    platforms=['OS Independent'],
    classifiers=classifiers,
    requires=INSTALL_REQUIRES,
    extras_require=EXTRA_REQUIRES,
    packages=find_packages(),
    package_dir={
        APP_NAME: APP_NAME,
    },
    data_files = media_files,
    zip_safe = False
)



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.contrib.sites.models import Site
from django import forms
from django.core import urlresolvers
from django.http import HttpResponse
from multilingual.admin import (
    MultilingualInlineAdmin, MultilingualModelAdmin, MultilingualModelAdminForm,
    MultilingualInlineModelForm
)
from simplegallery.models import Gallery, Image

class ReadOnlyLinkWidget(forms.Widget):
    def render(self, name, value, attrs=None):
        if value:
            return mark_safe(u'<a href="%s" onclick="return showRelatedObjectLookupPopup(this);">%s</a>' % (value, _('edit')))
        else:
            return u''

class ImageInlineForm(MultilingualInlineModelForm):
    admin_edit_url = forms.URLField(label=_('detail edit'), required=False,widget=ReadOnlyLinkWidget)
    def __init__(self, *args, **kwargs):
        super(ImageInlineForm, self).__init__(*args, **kwargs)
        if self.instance and self.instance.id:
            self.fields['admin_edit_url'].initial = urlresolvers.reverse('admin:simplegallery_image_change', args=(self.instance.id,))
    class Meta:
        model = Image
    
class ImageInline(admin.TabularInline):#MultilingualInlineAdmin):
    model = Image
    form = ImageInlineForm
    num_in_admin = 20 
    extra = 4 
    raw_id_fields = ('image',) # workaround... because otherwise admin will render an "addlink" after the field
    fields = ('image','ordering','admin_edit_url',)
    #readonly_fields = ('admin_edit_url',)
    def queryset(self, request):
        return self.model._default_manager.all()
    def edit_detail_link(self, obj):
        return '<a href="#">go bronkos!</a>'

class GalleryAdminForm(MultilingualModelAdminForm):
    current_request = None
    class Meta:
        model = Gallery
        
    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)
        data = kwargs.get('data', None)
        if instance:
            base_groups = [g.pk for g in instance.groups.all()]
        elif data:
            base_groups = data.get('groups', [])
        else:
            base_groups = []
        initial = kwargs.get('initial') or {}
        initial.update({'groups': [g.pk for g in self.current_request.user.groups.all()] + base_groups})
        kwargs['initial'] = initial
        super(GalleryAdminForm, self).__init__(*args, **kwargs)
        
    def clean_groups(self):
        groups = self.cleaned_data['groups']
        if not (self.current_request.user.groups.filter(pk__in=groups).count() or
                self.current_request.user.is_superuser):
            raise forms.ValidationError("You must choose at least one group you are in.")
        return groups
    

class GalleryAdmin(MultilingualModelAdmin):
    form = GalleryAdminForm
    inlines = [
        ImageInline,
    ]
    list_display = ('name', 'description', 'display_groups')
    search_fields = ('name', 'translations__title','translations__description',)
    # using ordering somehow results in double querysets
#    ordering = ('translations__title', )
    use_fieldsets = (
        (None, {
            'fields': ('name',),
        }),
        ('Language Dependent', {
            'fields': ('title', 'description'),
        }),
        ('Groups', {
            'classes': ('collapse',),
            'fields': ('groups',),
        }),
    )

    def display_groups(self, obj):
        return ', '.join([str(g) for g in obj.groups.all()])
    
    def queryset(self, request):
        qs = super(GalleryAdmin, self).queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(groups__in=request.user.groups.all()).distinct()
    
    def get_form(self, request, obj=None, **kwargs):
        form = super(GalleryAdmin, self).get_form(request, obj=None, **kwargs)
        form.current_request = request
        return form

class ReadOnlyImageWidget(forms.Widget):
    def render(self, name, value, attrs=None):
        if value:
            return mark_safe(u'<img src="%s" alt="" />' % (value,))
        else:
            return u''

class ImageDetailForm(MultilingualModelAdminForm):
    image_preview = forms.Field(label=_('image'), required=False, widget=ReadOnlyImageWidget)
    def __init__(self, *args, **kwargs):
        super(ImageDetailForm, self).__init__(*args, **kwargs)
        choices = [(s.id, s.name) for s in Site.objects.all()]
        self.fields['drop_up_links'].widget.choices = choices
        if self.instance and self.instance.id and self.instance.image:
            self.fields['image_preview'].initial = self.instance.image.icons.get('64','')

class ImageDetailAdmin(MultilingualModelAdmin):
    form = ImageDetailForm
    use_fieldsets= (
        (None, {'fields': ('gallery','image_preview',)}),
        (_('links'), {'fields': ('page_link','free_link',)}),
        (None, {'fields': ('title','description',)}),
        (_('advanced'), {'fields': ('drop_up_links',),'classes': ('collapse',),}),
    )
    readonly_fields = ('gallery',)
    filter_horizontal = ('drop_up_links',)
    def response_change(self, request, obj):
        if not request.POST.get("_continue"):
            return HttpResponse('<script type="text/javascript">window.close();</script>')
        return super(ImageDetailAdmin, self).response_change(request, obj)
    def has_add_permission(self, request):
        '''
        Can only be added in the context of a gallery
        '''
        return False
    def has_delete_permission(self, request, obj=None):
        '''
        Can only be deleted in the context of a gallery
        '''
        return False
    def get_model_perms(self, request):
        '''
        The image change view should only be accessable from the edit link
        on the inlines of the gallery change view.
        '''
        return {
            'add': False,
            'change': False,
            'delete': False,
        }

admin.site.register(Gallery, GalleryAdmin)
admin.site.register(Image, ImageDetailAdmin)